/*
Class holding details of a list of Adducts 
Based on Pure Appl.Chem., Vol.75, No.6, pp.683800, 2003
and
Rapid Commun. Mass Spectrom. 2007; 21: 2538-9
Checked against Fiehn lab web page and discrepancies noted:
http://fiehnlab.ucdavis.edu/staff/kind/Metabolomics/MS-Adduct-Calculator/
DPL 31.07.2011
 */

public class AdductList 
{
	private Adduct[] adductSet;
	private final int ADDUCT_LEN = 100;	// could make this expanding or hard code to size
	private int numAdducts = 0;			// number of Adducts in array	

	// Masses of adduct ions
	double hydrogenIon = 1.007276;
	double sodiumIon = 22.989222;			// Fiehn lab - 22.989218
	double potassiumIon = 38.963158;
	double ammoniumIon = 18.033825;			// Fiehn lab - 18.033823
	double water = 18.010565;				// Fiehn lab appearst to use val of 18.033563
	double chlorideIon = 34.969401;
	double bromideIon = 78.918887;
	double formicAcid = 46.005479;
	double aceticAcid = 60.021129;
	double tfa = 113.992864;			// trifluoroacetic acid
	double acn = 41.026549;				// acetonitrile: Fiehn lab - 41.026547
	double isopropanol = 60.057515;		// Fiehn lab differs at third decimal place!
	double methanol = 32.026215;		// Fiehn lab - 32.026213
	double dmso = 78.013935;			// dimethylsulphoxide: Fiehn lab 78.0139440
	
	public AdductList() 
	{
		adductSet = new Adduct[ADDUCT_LEN];
		populateSet();
	}

	// method to add Adducts to array
	// NB Unicode superscript plus = &#x207a; superscaript minus with &#x207b; subscript 2 = &#x2082; subscript 4 = &#x2084;
	public void populateSet() 
	{
		// +ve mode
		adductSet[numAdducts] = new Adduct(0, hydrogenIon, 1.0, true, true, "+H&#x207a;", "+ H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(1, sodiumIon, 1.0, true, true, "+Na&#x207a;", "+ Na&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(2, potassiumIon, 1.0, true, true, "+K&#x207a;", "+ K&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(3, ammoniumIon, 1.0, true, true, "+NH&#x2084;&#x207a;", "+ NH&#x2084;&#x207a;"); numAdducts++;		
		
		adductSet[numAdducts] = new Adduct(4, hydrogenIon, 0.5, true, false, "+2H&#x207a;", "+ 2H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(5, hydrogenIon, 0.333333, true, false, "+3H&#x207a;", "+ 3H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(6, sodiumIon, 0.5, true, false, "+2Na&#x207a;", "+ 2Na&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(7, sodiumIon, 0.333333, true, false, "+3Na&#x207a;", "+ 3Na&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(8, (hydrogenIon + sodiumIon)/2, 0.5, true, false, "+H&#x207a;+Na&#x207a;", "+ H&#x207a; + Na&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(9, (hydrogenIon + potassiumIon)/2, 0.5, true, false, "+H&#x207a;+K&#x207a;", "+ H&#x207a; + K&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(10, (hydrogenIon + ammoniumIon)/2, 0.5, true, false, "+H&#x207a;+NH&#x2084;&#x207a;", "+ H&#x207a; + NH&#x2084;&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(11, (2*hydrogenIon + sodiumIon)/3, 0.333333, true, false, "+2H&#x207a;+Na&#x207a;", "+ 2H&#x207a; + Na&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(12, (hydrogenIon + 2*sodiumIon)/3, 0.333333, true, false, "+H&#x207a;+2Na&#x207a;", "+ H&#x207a; + 2Na&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(13, (2*sodiumIon - hydrogenIon), 1.0, true, false, "+2Na&#x207a;-H&#x207a;", "+ 2Na&#x207a; - H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(14, (2*potassiumIon - hydrogenIon), 1.0, true, false, "+2K&#x207a;-H&#x207a;", "+ 2K&#x207a; - H&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(15, hydrogenIon, 2.0, true, false, "+H&#x207a;/2", "2X + H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(16, sodiumIon, 2.0, true, false, "+Na&#x207a;/2", "2X + Na&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(17, potassiumIon, 2.0, true, false, "+K&#x207a;/2", "2X + K&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(18, ammoniumIon, 2.0, true, false, "+NH&#x2084;&#x207a;/2", "2X + NH&#x2084;&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(19, (3*water + 2*hydrogenIon)/2, 1.0, true, false, "+3H&#x2082;O+2H&#x207a;", "+ 3H&#x2082;O + 3H&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(20, (acn + hydrogenIon), 1.0, true, false, "ACN+H&#x207a;", "+ Acetonitrile + H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(21, (isopropanol + hydrogenIon), 1.0, true, false, "Iso+H&#x207a;", "+ Isopropanol + H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(22, (methanol + hydrogenIon), 1.0, true, false, "MeOH+H&#x207a;", "+ Methanol + H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(23, (dmso + hydrogenIon), 1.0, true, false, "DMSO+H&#x207a;", "+ DMSO + H&#x207a;"); numAdducts++;			
		
		adductSet[numAdducts] = new Adduct(24, (isopropanol + hydrogenIon + sodiumIon), 0.5, true, false, "Iso+H&#x207a;+Na&#x207a;", "+ Isopropanol + H&#x207a; + Na&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(25, (hydrogenIon + acn/2), 0.5, true, false, "ACN+2H&#x207a;", "+ ACN + 2H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(26, (hydrogenIon + acn), 0.5, true, false, "2ACN+2H&#x207a;", "+ 2ACN + 2H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(27, (hydrogenIon + 3*acn/2), 0.5, true, false, "3ACN+2H&#x207a;", "+ 3ACN + 2H&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(28, (acn + sodiumIon), 1.0, true, false, "ACN+Na&#x207a;", "+ Acetonitrile + Na&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(29, (2*acn + hydrogenIon), 1.0, true, false, "2ACN+H&#x207a;", "+ 2Acetonitrile + H&#x207a;"); numAdducts++;

		adductSet[numAdducts] = new Adduct(30, (acn + hydrogenIon), 2.0, true, false, "ACN+H&#x207a;/2", "2X + ACN + H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(31, (acn + sodiumIon), 2.0, true, false, "ACN+Na&#x207a;/2", "2X + ACN + Na&#x207a;"); numAdducts++;		
		
		// -ve mode
		adductSet[numAdducts] = new Adduct(32, -1*hydrogenIon, 1.0, false, true, "-H&#x207a;", "- H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(33, -1*(hydrogenIon+water), 1.0, false, true, "-H&#x2082;O-H&#x207a;", "- H&#x207a; - H&#x2082;O"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(34, chlorideIon, 1.0, false, false, "+Cl&#x207b;", "+ Cl&#x207b;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(35, bromideIon, 1.0, false, false, "+Br&#x207b;", "+ Br&#x207b;"); numAdducts++;	
			
		adductSet[numAdducts] = new Adduct(36, -1*hydrogenIon, 0.5, false, false, "-2H&#x207a;", "- 2H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(37, -1*hydrogenIon, 0.333333, false, false, "-3H&#x207a;", "- 3H&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(38, -1*hydrogenIon, 2.0, false, false, "-H&#x207a;/2", "2X - H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(39, -1*hydrogenIon, 3.0, false, false, "-H&#x207a;/3", "3X - H&#x207a;"); numAdducts++;
		
		adductSet[numAdducts] = new Adduct(40, (sodiumIon - 2*hydrogenIon), 1.0, false, false, "+Na&#x207a;-2H&#x207a;", "+ Na&#x207a; - 2H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(41, (potassiumIon - 2*hydrogenIon), 1.0, false, false, "+K&#x207a;-2H&#x207a;", "+ K&#x207a; - 2H&#x207a;"); numAdducts++;
				
		adductSet[numAdducts] = new Adduct(42, (formicAcid - hydrogenIon), 1.0, false, false, "Form-H&#x207a;", "+ Formic Acid - H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(43, (aceticAcid - hydrogenIon), 1.0, false, false, "Acet-H&#x207a;", "+ Acetic Acid - H&#x207a;"); numAdducts++;		
		adductSet[numAdducts] = new Adduct(44, (tfa - hydrogenIon), 1.0, false, false, "TFA-H&#x207a;", "+ TFA - H&#x207a;"); numAdducts++;
						
		adductSet[numAdducts] = new Adduct(45, (formicAcid - hydrogenIon), 2.0, false, false, "Form-H&#x207a;/2", "2X + Formic Acid - H&#x207a;"); numAdducts++;
		adductSet[numAdducts] = new Adduct(46, (aceticAcid - hydrogenIon), 2.0, false, false, "Acet-H&#x207a;/2", "2X + Acetic Acid - H&#x207a;"); numAdducts++;
	}
	
	// reset all adducts to unselected (should be done ea run)
	public void deselectAdducts()
	{
		for (int i=0; i<numAdducts; i++)
		{
			adductSet[i].setSelected(false);
		}
	}
	
	// find Adduct by ID
	public Adduct findByID(int id)
	{
		for (int i=0; i<numAdducts; i++)
		{
			if(adductSet[i].getID() == id)
			{
				return adductSet[i];
			}
		}
		return null;
	}
	
	// returns number of adducts that have been flagged as selected
	public int getNumSelected()
	{
		int count = 0;
		for (int i=0; i<numAdducts; i++)
		{
			if(adductSet[i].isSelected() == true)
			{
				count++;
			}
		} 
		return count;
	}

	// -- accessor methods ---//

	public int getSize() 
	{
		return numAdducts;
	}

	public Adduct getAdduct(int i) 
	{
		return adductSet[i];
	}

}
