/*
Class holding details of a Kegg map: name, id and array of Metabolites it contains
together with information on number of these present in a given situation
Renamed from Pathway 14.04.10
DPL 02.09.2011
 */

public class Map 
{
	private String mapName;				// name of map
	private String mapID;				// kegg ID of map (includes suffix "map")
	private Metabolite[] metabSet;
	private final int MET_LEN = 200;	// should make this expanding !

	private int numMetabolites = 0;		// number of metabolites in full map		
	private int numPresent = 0;			// number of metabolites present in expt data

	private int duplicatesTotal = 0;	// total number of duplicate metabolites in Map
	private int duplicatesPresent = 0;	// number of metabolites present that are duplicates

	public Map(String name, String id) 
	{
		mapName = name;
		mapID = id;
		metabSet = new Metabolite[MET_LEN];
	}

	// constructor for use by sortMe() method - why?
	public Map()
	{
		mapName = "";
		mapID = "";
		metabSet = new Metabolite[MET_LEN];
	}

	// method to add Metabolite to array
	public void addMetabolite(Metabolite met) 
	{
		metabSet[numMetabolites] = met;
		numMetabolites++;
	}

	// method to update population
	public void incrementPresent() 
	{
		numPresent++;
	}
	
	// Determine and set degeneracy of formula of each metbolite of pathway present in expt
 	public void setDegeneracy()
	{
		setMapDegenOrdinals();						// sets an ordinal for occurrence on map (for duplicates)
		
		// traverse metabSet and assign a degeneracy for each formula, avoiding counting duplicates twice
		for (int i = 0; i < numMetabolites; i++)
		{
			// Only process metabolite if present
			if(metabSet[i].isPresent())
			{
				String formula = metabSet[i].getFormula();	// formula of this metabolite
				String name = metabSet[i].getName();		// name of this metabolite to check duplicates on map
				int degen = 0;	// initialize as 1, so can traverse and reject identities as duplicates
				for(int j = 0; j < numMetabolites; j++)
				{
					if(metabSet[j].isPresent() && metabSet[j].getFormula().equals(formula) &&
								metabSet[j].getMapDegenOrd() == 1)
							//!metabSet[j].getName().equals(name) && metabSet[j].getMapDegenOrd() == 1)
					{
						degen++;
					}
				}
				metabSet[i].setDegenPresent(degen);		
			}
		}
	}
	
	// sets an ordinal for occurrence on map (for duplicates)
	public void setMapDegenOrdinals()
	{
		// traverse metabSet and zero each map degeneracy ordinal
		for (int k = 0; k < numMetabolites; k++)
		{
			metabSet[k].setMapDegenOrd(0);
		}
		// traverse again and assign ordinal
		for (int i = 0; i < numMetabolites; i++)
		{
			// for each unassigned case
			if(metabSet[i].getMapDegenOrd() == 0)
			{
				int ordinal = 1;
				metabSet[i].setMapDegenOrd(ordinal);	// set it to 1
				ordinal++;
				// and go through rest of list and set any others
				for (int j = i+1; j < numMetabolites; j++)
				{
					if(metabSet[j].getName().equals(metabSet[i].getName()))
					{
						metabSet[j].setMapDegenOrd(ordinal);
						ordinal++;
					}
				}
			}
		}	
	}
	
	// selection sort of Metabolite array in ascending alphabetical order of Compound name 
	public void sort()
	{
		Metabolite lowest;		// holder for Metabolite with lowest alphabetical name
		int lowestPos;
		
		for (int i=0; i<numMetabolites-1; i++)	// do series of runs
		{
			for (int j=i+1; j<numMetabolites; j++)	// for ea run process list
			{
				lowest = metabSet[i];	// first of unsorted assigned to lowest
				lowestPos = i;
				
				if( (lowest.getName()).compareTo((metabSet[j].getName()) ) > 0)
				{
					lowestPos = j;
				}
				
				lowest = metabSet[lowestPos];
				metabSet[lowestPos] = metabSet[i]; 		// shift current first
				metabSet[i] = lowest;					// replace
			}
		}
	}

	// -- accessor methods ---//

	public String getMapName() 
	{
		return mapName;
	}

	public String getMapID() 
	{
		return mapID;
	}
	
	// returns ID with 'map' removed
	public String getMapIDnum()
	{
		return mapID.substring(3, mapID.length());
	}

	public int getNumPresent() 
	{
		return numPresent;
	}

	public int getNumMetabolites() 
	{
		return numMetabolites;
	}

	public Metabolite getMetabolite(int i) 
	{
		return metabSet[i];
	}
	
	// -- duplicate-free accessor methods for listing count ---//
	
	// returns number of unique metabolite names in Map
	public int getUniqueMetabolites() 
	{
		// First Sort array by Compound name
		sort();
		// Next go along summing unique names
		int uniqueMetabNum = 0;
		Metabolite previous = null;			// Metabolite at previous pos
		Metabolite current = null;			// Metabolite at current pos
		
		for (int i=0; i<numMetabolites; i++)
		{
			current = metabSet[i];
			if(previous == null)		// first
			{
				uniqueMetabNum++;
			}
			else if(!current.getName().equals(previous.getName()))
			{
				uniqueMetabNum++;
			}
			previous = current;
		}		
		return uniqueMetabNum;
	}
	
	// returns number of unique metabolite names in Map *present in Expt*
	public int getUniquePresent()
	{	
		// First Sort array by Compound name
		sort();
		// Next go along summing unique names of Metabolites present in Map
		int uniquePresentNum = 0;
		Metabolite previous = null;			// Metabolite Present at previous pos
		Metabolite current = null;			// Metabolite Present at current pos
	
		for (int i=0; i<numMetabolites; i++)
		{
			if(metabSet[i].isPresent())	// check this metabolite is present
			{
				current = metabSet[i];
				if(previous == null)		// first present
				{
					uniquePresentNum++;
				}
				else if(!current.getName().equals(previous.getName()))
				{
					uniquePresentNum++;
				}
				previous = current;				
			}
			// else go on to check next metabolite
		}
		return uniquePresentNum;
	}
}
