/*
Class holds details for a Metabolite.
This is not a pure abstract representation of a Metabolite, 
but models its occurrence in an experiment and in the context of a particular Map
So there will be a separate version for each map (to accomodate different glyph posns)
DPL 02.09.2011
 */
 
import java.net.*;
import java.io.*;

public class Metabolite 
{
	// instance variables for Kegg properties
	String name;			// CompoundName in db
	String formula;			// Formula in db
	int degeneracy;			// Degeneracy of Formula from DB - 1 if unique
	String id;				// CompoundID in db (e.g. C14161)
	double mass;			// MIM in db
	String metacycUID;		// MetaCyc UID (may be null)
	
	// instance variables wrt to presence in containing pathway in input data
	boolean present;		// present in containing pathway
	int degenPresent = 0;	// degeneracy of formula in metabolites found for pathway
	int mapDegenOrd = 0;	// this is a unique integer for a metabolite on a map
							// if present only once it will be 1, otherwise 1 or 2 or ...
	
	// instance variables related to Expt
	boolean isExpt = false;	// whether holding data from comparative expt
	StringBuffer adducts;	// holds details of adducts
	int[] values;			// holds values for a set of conditions
	int[] errors;			// holds errors for a set of conditions
	String[] conditions;	// holds details of conditions - stupid in this class as repitious!
	int length;				// array length (No of expt conditions)
	
	// instance variables related to position in map
	String glyphShape;
	int glyphX;
	int glyphY;
	int glyphWidth;
	int glyphHeight;
	
	int baseCondition = 0;	// Condition number (counting from 1) for baseline (0 = no cf)
	int exptCondition = 0;	// Condition number (counting from 1) for expt (0 = no cf.)
	int change = 0;			// +3 to +1 : increase, -1 to -3 : decrease, 0 : no change
	
	double up3, up2, up1, down1, down2, down3;		// cut-off values
	
	ExptSettings settings;

	public Metabolite(String name, String formula, int degeneracy, String id, double mass, 
								String glyphShape, int glyphX, int glyphY, int glyphWidth, int glyphHeight,
								int numConditions) 
	{
		this.name = name;
		this.formula = formula;
		this.degeneracy = degeneracy;
		this.id = id;
		this.mass = mass;
		
		this.glyphShape = glyphShape;
		this.glyphX = glyphX;
		this.glyphY = glyphY;
		this.glyphWidth = glyphWidth;
		this.glyphHeight = glyphHeight;
		
		length = numConditions;		
		if(length > 1)
		{
			isExpt = true;
			values = new int[numConditions];
			errors = new int[numConditions];
			conditions = new String[numConditions];
		}
		else
		{
			isExpt = false;		
		}
		
		present = false;		
		adducts = new StringBuffer("(");
	}
	
	// set method for use if Expt
	public void setSettings(ExptSettings eSettings)
	{
		if(eSettings != null)
		{
			settings = eSettings;
			this.baseCondition = settings.getBaseCondition();
			this.exptCondition = settings.getExptCondition();
			this.up3 = settings.getUp3();
			this.up2 = settings.getUp2();
			this.up1 = settings.getUp1();
			this.down1 = settings.getDown1();
			this.down2 = settings.getDown2();
			this.down3 = settings.getDown3();	
		}	
	}
	
	// controller method for adding new metabolite details - no adduct version
	public void add(int[] vals, int[] errs, String[] conds)
	{
		add(vals, errs, conds, null);
	}
	
	// controller method for adding new metabolite details
	public void add(int[] vals, int[] errs, String[] conds, String adduct)
	{
		// Check whether results already added for this metabolite
		boolean atStart = true;
		if(conditions[0] !=null)
		{
			atStart = false;
		}
		
		// if so check if new data contains a higher value
		boolean  changeVals = true;
		if(atStart == false)
		{
			// Find highest value in current set
			int currentHigh = 0;
			for(int i=0; i<values.length; i++)
			{
				if(values[i] > currentHigh)
				{
					currentHigh = values[i];
				}
			}
			
			// Find highst value in new set
			int newHigh = 0;
			for(int j=0; j<vals.length; j++)
			{
				if(vals[j] > newHigh)
				{
					newHigh = vals[j];
				}
			}	
			
			// compare and flag not to change, if appropriate
			if(currentHigh > newHigh)
			{
				changeVals = false;
			}		
		}
		
		// Call change methods if appropriate
		if(atStart || changeVals)
		{
			addValues(vals);
			addErrors(errs);
			addConditions(conds);
			if(adduct != null)
			{
				addAdduct(adduct);
			}
		}
	}
	
	// updates the values array by adding data from a new peak
	private void addValues(int[] vals)
	{
		for (int i =0; i<length; i++)
		{
			values[i] = vals[i];
		}
	}
	
	// updates the errors array by adding data from a new peak
	private void addErrors(int[] errs)
	{
		for (int i =0; i<length; i++)
		{
			errors[i] = errs[i];
		}
	}
	
	private void addConditions(String[] conds)
	{
		for (int i =0; i<length; i++)
		{
			conditions[i] = URLEncoder.encode(conds[i]) ;
		}
	}
	
	// adds adduct to StringBuffer of adducts
	private void addAdduct(String adduct)
	{
		// clear any previous content of the sb, but leave opening parenthesis
		if(adducts.length() > 1)
		{
			adducts.delete(1, adducts.length());
		}
		adducts.append(" " + adduct);
	}
	
	public void addAdductSimple(String adduct)
	{
		adducts.append(" " + adduct);	
	}
	
	// returns an image tag pointing to the mscompare servlet - change from hard-coding
	public String getComparisonImage()
	{
		String bar = null;		// class indicative of colour coding
		calculateChange();		// make sure that state change has been calculated
		// calculate colour-coding class on the basis of degree and direction of change
		if(change == -3)
		{bar = "bar3n";}
		else if(change == -2)
		{bar = "bar2n";}
		else if(change == -1)
		{bar = "bar1n";}
		else if(change == 0)
		{bar = "bar0";}
		else if(change == 1)
		{bar = "bar1p";}
		else if(change == 2)
		{bar = "bar2p";}
		else if(change == 3)
		{bar = "bar3p";}
		else
		{bar = "bar";}
		
		StringBuffer sb = new StringBuffer("<span class=\"" + bar + "\"><a href=\"javascript:openNewWindow('/mscompare/mscompare.html?metabolite=");
		sb.append(URLEncoder.encode(escapeQuote(name)) + "&amp;numConditions=" + length);
		
		sb.append("&amp;condNames=");
		for(int i=0; i<length-1; i++)
		{
			sb.append(conditions[i] + "%2C");
		}
		sb.append(conditions[length-1]);
		
		sb.append("&amp;condValues=");		
		for(int i=0; i<length-1; i++)
		{
			sb.append(values[i] + "%2C");
		}
		sb.append(values[length-1]);
		
		sb.append("&amp;condErrors=");
		for(int i=0; i<length-1; i++)
		{
			sb.append(errors[i] + "%2C");
		}
		sb.append(errors[length-1]);
		
		sb.append("','" + URLEncoder.encode(escapeQuote(name)) + "',620,420)\">G</a></span>");
		return sb.toString();
	}
	
	// returns imagemap area link
	public String getAreaSpec()
	{
		calculateChange();		// make sure that state change has been calculated

		StringBuffer sb = new StringBuffer("<area shape=\"" + glyphShape + "\" ");
		if(glyphShape.equals("circle"))
		{
			sb.append("coords=\"" + glyphX + "," + glyphY + "," + glyphWidth + "\" ");
		}
		else
		{
			sb.append("coords=\"" + glyphX + "," + glyphY + "," + glyphWidth + "," + glyphHeight + "\" ");		
		}
		
		sb.append("href=\"javascript:openNewWindow('/mscompare/mscompare.html?metabolite=");
		sb.append(URLEncoder.encode(escapeQuote(name)) + "&amp;numConditions=" + length);
		
				sb.append("&amp;condNames=");
		for(int i=0; i<length-1; i++)
		{
			sb.append(conditions[i] + "%2C");
		}
		sb.append(conditions[length-1]);
		
		sb.append("&amp;condValues=");		
		for(int i=0; i<length-1; i++)
		{
			sb.append(values[i] + "%2C");
		}
		sb.append(values[length-1]);
		
		sb.append("&amp;condErrors=");
		for(int i=0; i<length-1; i++)
		{
			sb.append(errors[i] + "%2C");
		}
		sb.append(errors[length-1]);
		
		
		sb.append("','" + URLEncoder.encode(escapeQuote(name)) + "',620,420)\" alt=\"\" />");
		return sb.toString();
	}
	
	// calculates and returns state (ie whether increase or decrease - cannot be done in constructor
	public int getState()
	{
		if(length > 1)
		{
			if(baseCondition > 0 && exptCondition > 0)
			{
				calculateChange();
			}	
			return change;
		}
		else
		{
			return 0;
		}
	}	
	
	// calculates whether expt has produced a change and of what magnitude
	private void calculateChange()
	{
		double base = (double) values[baseCondition-1];
		double expt = (double) values[exptCondition-1];
		
		// start with highest increase - then catch successively smaller increases in next two step
		if(expt >= base*up3)
		{
			change = 3;
		}
		else if(expt >= base*up2)
		{
			change = 2;
		}
		else if(expt >= base*up1)
		{
			change = 1;
		}
		// now check for highest decrease (i.e. smallest values) - then catch successively smaller decreases in next two step
		else if(expt <= base*down3)
		{
			change = -3;
		}
		else if(expt <= base*down2)
		{
			change = -2;
		}
		else if(expt <= base*down1)
		{
			change = -1;
		}
		// should be > down1x and < up1x - could check directly if required for debugging
		else
		{
			change = 0;	
		}
	}
	
		// -- accessor methods ---//

	public void setPresent(boolean found) 
	{
		present = found;
	}
	
	public void setDegenPresent(int num) 
	{
		degenPresent = num;
	}
	
	public void setMapDegenOrd(int num) 
	{
		mapDegenOrd = num;
	}
	
	// sets Metacyc UID
	public void setMetacycUID(String uid)
	{
		metacycUID = uid;
	}
	
	// careful: can be null!
	public String getMetacycUID()
	{
		return metacycUID;
	}

	public String getName() 
	{
		return name;
	}

	public String getFormula() 
	{
		return formula;
	}
	
	public int getFormDegen() 
	{
		return degeneracy;
	}
	
	public int getMapDegenOrd()
	{
		return mapDegenOrd;
	}

	public String getID() 
	{
		return id;
	}
	
	// Glyph details
	public String getGlyphShape() 
	{
		return glyphShape;
	}	
	public int getGlyphX()
	{
		return glyphX;
	}
	public int getGlyphY()
	{
		return glyphY;
	}
	public int getGlyphWidth()
	{
		return glyphWidth;
	}
	public int getGlyphHeight()
	{
		return glyphHeight;
	}	
		
	// returns ID of metabolite with "cpd:" prefix for Kegg web service methods
	public String getLongID()
	{
		return "cpd:" + id;
	}
	
	public double getMass()
	{
		return mass;
	}

	public boolean isPresent() 
	{
		return present;
	}
	
	// returns No of instances of formula in metabolites found for pathway
	public int getDegenPresent() 
	{
		return degenPresent;
	}
	
	// returns whether more than one instance of formula in metabolites found for pathway
	public boolean isDegenPresnt()
	{
		if(degenPresent > 1)
		{
			return true;
		}
		else
		{
			return false;	
		}
	}
		
	public String getAdducts()
	{
		adducts.append(" )");		// finish off parenthetical string
		return adducts.toString();
	}

	// looks for single quote in String and replaces it by \'
	public String escapeQuote(String name)
	{
		if(name.indexOf("'") != 0)
		{
			StringBuffer sb = new StringBuffer("");
			for (int i=0; i<name.length();i++)
			{
				char c = name.charAt(i);
				if(c == '\'')
				{
					sb.append( "\\" + c);
				}
				else
				{
					sb.append(c);
				}
			}
			return sb.toString();		
		}
		else
		{
			return name;
		}
	}
}
