/*
	Class to create Buffered Image of Graph
	DPL 28.02.2011
*/		

import java.awt.*;
import java.awt.image.*;

public class CreateImage
{
	String metabolite;		// name of metabolite
	int numConditions;		// number of conditions
	String[] names;			// array of names of conditions
	int[] values;			// array of condition values
	int[] errors;			// array of error values
	
	// fonts
	private Font titleFt = new Font("SansSerif", Font.BOLD,14);
	private Font axisFt = new Font("SansSerif", Font.PLAIN,10);	
	private Font ssFt = new Font("SansSerif", Font.PLAIN,8);
	
	// overall dimensions - these need to be hard coded - could vary with number of conds, I suppose
	private int imgWidth = 600;
	private int imgHeight = 400;
	
	// layout gutters and spacings
	private int gutter = 20;	// minimum standoff of graphics from edge
	
	private int topSpace;		// space between top gutter and graph
	private int botSpace;		// space between bottom gutter and graph
	private int lSpace;			// space between left gutter and graph
	private int rSpace;			// space between right gutter and graph
	
	// graph dimensions
	int oriX;
	int oriY;
	int graphWidth;
	int graphHeight;
	
	int tickWidth = 5;			// hard-code width of tick on y axis
	
	// bar dimensions
	int barWidth;		// width of bar
	int spaceWidth;		// width of space between bars - half of this at start and (poss) end
	
	// value variables	
	double scaleMax;	// value of scale max x 10^-power
	int power;			// powers of 10 scale values are multiplied by
	double factor;		// factor to convert values to pixels
	
	// etc
	Color myBlue;
	
	// Buffered Image		
	BufferedImage buffImage;		// the image to be created

	public CreateImage(String metabolite, int numConditions, 
							String[] names, int[] values, int[] errors)
	{	
		this.metabolite = metabolite;
		this.numConditions = numConditions;
		this.names = names;
		this.values = values;
		this.errors = errors;
		
		myBlue = new Color(73, 136, 212);
		
		buffImage = createPngImage();		// generate the image
	}
	
    // Master drawing method: calls subsidiary methods and finally returns generated image	
	public BufferedImage createPngImage()
	{
        // Create a buffered image in which to draw
        BufferedImage bufferedImage = new BufferedImage(imgWidth, imgHeight, 
        										BufferedImage.TYPE_INT_RGB);
    
        // Create a graphics contents on the buffered image
        Graphics2D g = bufferedImage.createGraphics();
        
        // fill area with white
        g.setColor(Color.white);
		g.fillRect(0, 0, imgWidth, imgHeight);
					
		drawFrame(g);
		drawTitle(g);
		drawGraph(g);
		drawYaxis(g);
		
		// Graphics context no longer needed so dispose of it
        g.dispose();
        
        return bufferedImage;
	}

	//------------------------ Subsidiary drawing methods ---------------------------//
			
	// draw rectangular frame of graph leaving gutter and text space
	private void drawFrame(Graphics g)
	{
		calculateStandoff(g);
		
		oriX = gutter + lSpace;
		oriY = gutter + topSpace;	
		graphWidth = imgWidth - 2*gutter - lSpace - rSpace;
		graphHeight = imgHeight - 2*gutter - topSpace - botSpace;
		
		g.setColor(Color.black);
		g.drawRect(oriX, oriY, graphWidth, graphHeight);
	}
	
	// Draws overall title: name of metabolite
	private void drawTitle(Graphics g)
	{
		g.setFont(titleFt);
		FontMetrics fm = g.getFontMetrics(titleFt);
		String title = ( metabolite.substring(0,1).toUpperCase() 
						+ metabolite.substring(1, metabolite.length()));
		int xTitle = oriX + (graphWidth - fm.stringWidth(title))/2;			
		int yTitle = gutter + fm.getAscent();
		g.setColor(Color.black);
		g.drawString( title, xTitle, yTitle);
	}
	
	// Draws bars for graph and x axis names
	private void drawGraph(Graphics g)
	{
		calculateBars();
		calculateScale();
			
		int barX = oriX + spaceWidth/2;
		for(int i=0; i<numConditions; i++)
		{
			// Draw rectangular bars
			g.setColor(myBlue);
			g.fillRect(barX, oriY + (int) (graphHeight - values[i]*factor) + 1, barWidth, (int) (values[i]*factor) );
			g.setColor(Color.black);
			g.drawRect(barX, oriY + (int) (graphHeight - values[i]*factor) + 1, barWidth, (int) (values[i]*factor) );
			
			// Draw error markings
			g.setColor(Color.black);
			int errorMax = (int) ((values[i] + errors[i])*factor);
			int errorMin = (int) ((values[i] - errors[i])*factor);

			g.drawLine((barX + 3*barWidth/8), (oriY + graphHeight - errorMax), 
										(barX + 5*barWidth/8), (oriY + graphHeight - errorMax));		// top	
			g.drawLine((barX + 3*barWidth/8), (oriY + graphHeight - errorMin), 
										(barX + 5*barWidth/8), (oriY + graphHeight - errorMin));		// bottom
			g.drawLine((barX + barWidth/2), (oriY + graphHeight - errorMax), 
										(barX + barWidth/2), (oriY + graphHeight - errorMin));			// join
										
			// Draw legends (names)
			g.setFont(axisFt);
			FontMetrics fm = g.getFontMetrics(axisFt);
			int xName = barX + barWidth/2 - fm.stringWidth(names[i])/2;			
			int yName = imgHeight - gutter - fm.getDescent();
			
			g.setColor(Color.black);
			g.drawString( names[i], xName, yName);			
			
			// increment barX
			barX = barX + barWidth + spaceWidth;
		}
	}	
		
	// draw a suitable number of ticks along Y axis together with values
	private void drawYaxis(Graphics g)
	{
		double stepSize = 0.0;	// spacing (*10-power) - chosen arbitraly
		
		if(scaleMax>4)
		{
			stepSize = 1.0;
		}
		else if (scaleMax>1.5)
		{
			stepSize = 0.5;
		}
		else
		{
			stepSize = 0.2;
		}
		
		double pxFactor = graphHeight/scaleMax;
		int interval = (int) (stepSize*pxFactor);	// interval between ticks in pixels
		int stepNo = (int)	(scaleMax/stepSize) + 1;	// number of ticks (starting from 0)
		
		g.setColor(Color.black);
		for(int i=0; i< stepNo; i++)
		{
			// tick
			g.drawLine(oriX, (oriY+graphHeight - i*interval), oriX-tickWidth, (oriY+graphHeight - i*interval));
			// number string
			String numString = (String.valueOf(stepSize*i)).substring(0,3);	// three chars including .
			g.setFont(axisFt);
			FontMetrics fm = g.getFontMetrics(axisFt);
			int xNum = oriX - tickWidth*2 - fm.stringWidth(numString);
			int yNum = (oriY+graphHeight - i*interval) + fm.getAscent()/2;
			g.drawString(numString, xNum, yNum);
		}	
		
		// Power factor
		String timesTen = "Ht \u00D7 10";
		String minusPower = "\u2013" + String.valueOf(power);	
		
		FontMetrics fma = g.getFontMetrics(axisFt);
		
		g.setFont(axisFt);		
		int xP = oriX - tickWidth*2 - fma.stringWidth(timesTen)/2;	// same as numbers
		int yP = imgHeight - gutter - fma.getDescent();				// same as legends
		g.drawString(timesTen, xP, yP);		
		
		g.setFont(ssFt);
		int xS = xP + fma.stringWidth(timesTen);		// end of 10x
		int yS = yP - fma.getAscent()/2;				// half way up ?
		g.drawString(minusPower, xS, yS);	
	}


	//--------------------------- Utility calculation methods ------------------------------//	
		
	// calculates standoff for frame to accomodate text
	private void calculateStandoff(Graphics g)
	{
		FontMetrics fmt = g.getFontMetrics(titleFt);
		topSpace = 20 + fmt.getAscent() + fmt.getDescent();
		
		FontMetrics fma = g.getFontMetrics(axisFt);
		botSpace = 20 + fma.getAscent() + fma.getDescent();	
		
		String leftNumeral = "0.0";
		int leftWidth = fma.stringWidth(leftNumeral);
		lSpace = leftWidth + 10 ;
		
		rSpace = 0;			// no extra space needed
	}
	
	// calculates width of bars and spaces wrt No conditions: max spacing - 2x bar width, min - 0
	private void calculateBars()
	{
		int minBar = 50;
		int maxBar = 75;
				
		if(numConditions*(3*maxBar) < graphWidth)	// need to use max for both
		{
			barWidth = maxBar;
			spaceWidth = 2*maxBar;
		}
		else
		{
			int testBar = graphWidth / (3*numConditions);
			if(testBar >= minBar)				// share in proportion if possible
			{
				barWidth = testBar;
				spaceWidth = 2*testBar;
			}
			else								// use minimum bar width and lower space
			{
				barWidth = minBar;
				spaceWidth = (graphWidth - numConditions*barWidth)/numConditions;
				if(spaceWidth < 0)					//	unlikely event of v.many conditions 
				{
					spaceWidth = 0;
					barWidth = graphWidth/numConditions;
				}
			}
		}
	}
	
	// Find highest bar and decide on what value to set as maximum, and hence set scale value
	private void calculateScale()
	{
		// Find maximum value + error
		int max = 0;
		for (int i=0; i<numConditions; i++)
		{
			int current = values[i] + errors[i];
			if(current > max)
			{
				max = current;
			}
		}
		
		// Find initial digit for max and power of 10
		boolean done = false;
		int digit = max;		// initial digit in max
		power = 0;				// number of powers of 10 this digit is x by in max
		if(digit < 10)
		{
			done = true;
		}
		while (!done)
		{
			digit = digit/10;
			power++;
			if(digit < 10)
			{
				done = true;
			}
		}

		// Make scaleMax 1 more than digit of max value
		scaleMax = digit + 1.0;
		
		// finally calculate factor for converting values to pixels
		factor = (double) graphHeight / (scaleMax * Math.pow(10, power));
	}
	
	//-----------------------------------------------------------------//
	
	// accessor method for final BufferedImage
	public BufferedImage getBufferedImage()
	{
		return buffImage;
	}
	
}